@@ -7,6 +7,7 @@
 #include <libgnomevfs/gnome-vfs-resolve.h>
 #include <libgnomevfs/gnome-vfs-dns-sd.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include "pygnomevfsbonobo.h"
 
 static GHashTable *monitor_hash;
@@ -477,7 +478,36 @@ pygvfs_truncate(PyObject *self, PyObject *args, PyObject *kwargs)
     }
 
     if (pygnome_vfs_result_check(result))
-	return NULL;
+        return NULL;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+pygvfs_create_symbolic_link(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "uri", "target", NULL };
+    PyObject *uri;
+    char *target;
+    GnomeVFSResult result;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs,
+                                     "Os:gnomevfs.create_symbolic_link", kwlist,
+                                     &uri, &target))
+        return NULL;
+    
+    if (!PyObject_TypeCheck(uri, &PyGnomeVFSURI_Type)) {
+        PyErr_SetString(PyExc_TypeError, "uri must be a gnomevfs.URI");
+        return NULL;
+    }
+
+    pyg_begin_allow_threads;
+    result = gnome_vfs_create_symbolic_link(pygnome_vfs_uri_get(uri), target);
+    pyg_end_allow_threads;
+        
+    if (pygnome_vfs_result_check(result))
+        return NULL;
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -615,6 +645,34 @@ pygvfs_exists(PyObject *self, PyObject *args, PyObject *kwargs)
     return PyInt_FromLong(exists);
 }
 
+static PyObject *
+pygvfs_get_file_mime_type(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = {"path", "fast", "suffix_only", NULL };
+    const char *path;
+    const char *mime;
+    gboolean fast = TRUE;
+    gboolean suffix = FALSE;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs,
+                                    "s|ii:gnomevfs.get_file_mime_type",
+                                    kwlist,
+                                    &path, &fast, &suffix))
+    return NULL;
+
+    if (fast)
+        mime = gnome_vfs_get_file_mime_type_fast(path, NULL);
+    else
+        mime = gnome_vfs_get_file_mime_type(path, NULL, suffix);
+
+    if (mime == NULL) {
+        Py_XINCREF(Py_None);
+        return Py_None;
+    }
+
+    return PyString_FromString(mime);
+}
+
 static PyObject *
 pygvfs_get_mime_type(PyObject *self, PyObject *args)
 {
@@ -2013,6 +2071,24 @@ _wrap_gnome_vfs_get_default_browse_domains(PyObject *self, PyObject *args, PyObj
     return retval;
 }
 
+
+static PyObject *
+_wrap_gnome_vfs_mime_type_get_equivalence(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "mime_type", "base_mime_type", NULL };
+    char *mime_type, *base_mime_type;
+    GnomeVFSMimeEquivalence result;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs,
+				     "ss:gnomevfs.mime_type_get_equivalence", kwlist,
+                                      &mime_type, &base_mime_type))
+	return NULL;
+
+    result = gnome_vfs_mime_type_get_equivalence(mime_type, base_mime_type);
+    return PyInt_FromLong(result);
+}
+
+
 static PyMethodDef pygnomevfs_functions[] = {
     { "create", (PyCFunction)pygvfs_create, METH_VARARGS|METH_KEYWORDS },
     { "get_file_info", (PyCFunction)pygvfs_get_file_info,
@@ -2024,9 +2100,12 @@ static PyMethodDef pygnomevfs_functions[] = {
       METH_VARARGS|METH_KEYWORDS },
     { "remove_directory", (PyCFunction)pygvfs_remove_directory,
       METH_VARARGS|METH_KEYWORDS },
+    { "create_symbolic_link", (PyCFunction)pygvfs_create_symbolic_link,
+      METH_VARARGS|METH_KEYWORDS },
     { "unlink", (PyCFunction)pygvfs_unlink, METH_VARARGS|METH_KEYWORDS },
     { "exists", (PyCFunction)pygvfs_exists, METH_VARARGS|METH_KEYWORDS },
     { "format_file_size_for_display", pyvfs_format_file_size_for_display, METH_VARARGS},
+    { "get_file_mime_type", (PyCFunction) pygvfs_get_file_mime_type, METH_VARARGS|METH_KEYWORDS},
     { "get_mime_type", (PyCFunction)pygvfs_get_mime_type, METH_VARARGS },
     { "get_mime_type_for_data", (PyCFunction)pygvfs_get_mime_type_for_data,
       METH_VARARGS },
@@ -2144,6 +2223,9 @@ static PyMethodDef pygnomevfs_functions[] = {
       METH_VARARGS|METH_KEYWORDS},
     { "get_default_browse_domains",
       (PyCFunction)_wrap_gnome_vfs_get_default_browse_domains, METH_NOARGS},
+    { "mime_type_get_equivalence",
+      (PyCFunction)_wrap_gnome_vfs_mime_type_get_equivalence,
+      METH_VARARGS|METH_KEYWORDS},
 
     { NULL, NULL, 0 }
     
@@ -2284,6 +2366,9 @@ register_constants(PyObject *module)
     regconst(PRIORITY_MIN);
     regconst(PRIORITY_MAX);
     regconst(PRIORITY_DEFAULT);
+    regconst(MIME_UNRELATED);
+    regconst(MIME_IDENTICAL);
+    regconst(MIME_PARENT);
 
 #undef regconst
 }
